function [A,P,Res] = greyca(X,A0,lambda,As,Aold);
% [A,P,Res] = greyca(X,A0,lambda,As,Aold);
% X:        data
% A0:       structure of restricted matrix
% lambda:   Can be a scalar or vector with lambda values
% As:       Matrix with zeros (A0 value not restrictive) and ones (A0 value
%           restrictive)
% Aold:     Old solution of A used as starting point for new calculation.
% A:        Final result for A (with last lambda)
% P:        Final result for P (with last lambda)
% Res:      Results, contains 
%           [lambda, 
%            ssq of X, (||X - AP||^2)
%            ssq of restriction, (||As*(A - A0)||^2)
%            total ssq, (||X - AP||^2 + lambda||As*(A - A0)||^2)
%           ]
%
% GreyCA starts with a pca on a data matrix X. The solution is rotated
% to minimize the ssq of the restriction. This solution in still within the
% PCA space and describes maximum variance.
% Then an ALS approach is used to minimize the following :
%
% Q = ||X - AP||^2 + lambda||As.*(A - A0)||^2
%
% by changing A, where A0 is a matrix with the attempted score values. Note
% that only nonzero elements of As are taken into account. (.* represents
% Hadamard product).
%
% restricted LS is used to calculate P with restriction that diag(PP') only
% consists of ones (thanks to Henk Kiers).
%
% JAW UVA-BDA, March 3, 2006

[I,J] = size(X);
[I,R] = size(A0);

As = sparse(As);
A0 = sparse(A0);
Ab = sparse(diag(As(:)));
Ab = sparse(Ab'*Ab);
Xv = X(:);
A0v = A0(:);A0v = sparse(A0v);
conv_c = [1E-6,50000];

if isempty(lambda)
    lambda = [0 0.001 0.003 0.01 0.03 0.1 0.3 1 3 10 30 100 300 1000 1000000];
end
nl = max(size(lambda));
Res = zeros(nl+1,4);

if isempty(Aold)
    %First do PCA on X 
    [U,S,V] = svds(X,R);
    A = U*S;clear U S;
    % Rotate PCA close to restricted solution A0
    [Ar] = rotatepca_jw(A0,A,As);
    P = Ar\X;
    Aold = Ar;
end

% Fit by PCA model or by Aold model
P = Aold\X;
A = (P'\X')';
E = X - A*P;
SSEpca = ssq(E);
SSErestr_pca = ssq(E)+lambda(1)*ssq(As.*(A0-A));
SSEdiff = SSErestr_pca;    
SSE = 1E6;
Res(1,:) = [0 SSEpca ssq(As.*(A0-A)),SSEpca];

% Here ALS starts to estimate A and P
for h = 1:nl
    kount = 0;
    %A = Aold;

    while (kount<15 | (SSEdiff>conv_c(1) & kount<conv_c(2)))
        kount = kount+1;
    
        % Estimate P
        P = (rls(X,A,P'))';

        % Estimate A
        F = (kron(P',speye(I))); F = sparse(F);
        FF = F'*F; FF = sparse(FF);
        FFDD = (FF+lambda(h)*Ab);FFDD = sparse(FFDD); clear FF
        Av = FFDD\(F'*Xv + lambda(h)*Ab*A0v);
        A = reshape(Av',I,R); 
        clear Av FFDD F

        % Check convergence
        E = X - A*P;
        B = As.*(A0-A);
        ssqE = ssq(E);
        ssqB = ssq(B);
        SSErestr_pca = ssqE+lambda(h)*ssqB;
        
        if kount > 5
        if SSE < SSErestr_pca
            disp('SSE goes up');
        end
        end
        SSEdiff = abs(SSE - SSErestr_pca)/SSE;
        SSE = SSErestr_pca;
        s=sprintf('%5.4g %8.5g %15.5g %15.5g %15.5g',lambda(h),kount,ssqE,full(ssqB),SSErestr_pca);
        disp(s);
    end
    
    Res(h+1,:) = [lambda(h),ssqE,ssqB,SSErestr_pca];
end